#!/usr/local/bin/perl


# short version
# CALL_TSK={MOD=P,TSK=61,
use Getopt::Std;

getopts( 'm:hHlx:pf:' );
$printingAllTasksProgram = 0;
$onlyPrintColors = 0;
$progNum = 1;
$mmddyy = "[0-9][0-9]\/[0-9][0-9]\/[0-9][0-9]";
$hhmmss = "[0-9][0-9]:[0-9][0-9]:[0-9][0-9]";
if ( $#ARGV == -1 ) {
    $printingAllTasksPrograms = 1;
}
if ( $opt_h || $opt_H ||  $ARGV[0] =~ /-(h|&)/i ) { 
	usage() ;
}
else {
    $search = $ARGV[0];
    if ( !($opt_l) ){ # no labels
		$plabel = "Program:";
        $tlabel = "Task:";
    } else {
		$plabel = "";
        $tlabel = "";
    }

    if ( $opt_x eq 'p' ){
		$search = "CALL_TSK={MOD=P,TSK=(" . $search . "),";
    }
    elsif ( $opt_x eq 'e' ){   #ets search, expect $opt_f to have a file.
		die "For ETS search, need ETS list filename specified with -f filename.\n" if (!defined ($opt_f));
		die "Filename \"$opt_f\" For ETS search does not exist.\n" if ( !-f $opt_f);
		my $etslist = read_ETS_list( $opt_f );
		die "ETS List has no ETS\'s?\n" if (!length($etslist));
		$search = "INIGET \\\(\\\'\\\[Magic_Logical_Names\\\]($etslist)\\\'";
        $printingAllTasksPrograms = 0;
	}
    elsif ( $opt_x eq 'c' ){
		$search = "COLOR=(" . $search . "),";
    }
    elsif ( $opt_x  eq 'f' ){
		$search = "^DB={FILE=" . $search . ",";
        if(    $opt_m eq 'w' ){	   $search .= ".+?MOD=W"	}
		elsif( $opt_m eq 'r' ){    $search .= ".+?MOD=R"	}
    }
    elsif ( $opt_x eq 't' ){
		$search = "^UPD=\{DATE=\"($mmddyy)\",TIME=\"($hhmmss)\"";
        $printingAllTasksPrograms = 0;
    }
    elsif ( $opt_x eq 'l' ){
		#	$search = "INIGET";
		$search = "INIGET \\\(\\\'\\\[Magic_Logical_Names\\\]([A-Z][A-Z][0-9]+)\\\'\\\)";
        $printingAllTasksPrograms = 0;
    }
    if ( $#ARGV == -1 && !(($opt_x eq 't')|| ($opt_x eq 'l'))){
		print " -h or -? returns help\n";
    } 
	else {
        print STDERR "using search string: ", $search, "\n";
    }
}

while( <STDIN>){	# reads string into $_
    if    ( /^(TSK|PRG)/ ){ 	$expect = $1; }
    elsif ( /^HDR={DESC=\"(.+[^\"])\",/  ){	#tests $_ for a match anywhere in string
        $desc = $1;
		if( $expect =~ /^T/ ){
			$task = $desc;	
			print "\t$tlabel" if $printingAllTasksPrograms ;
		}
		else {
			$prog = $desc;  $task = '';	    
			print $plabel,$progNum++, " " if $printingAllTasksPrograms ;
		}
		print $desc . "\n"  unless !$printingAllTasksPrograms;
    }
	else {
		undef($1); undef($2);
		if ( defined( $search ) &&  /$search/i ) {
            if( $opt_x ne 't' || $opt_p ne '1' || $task eq '' ){
				print $prog, "\t", $task unless $onlyPrintMatch ;
				if( defined( $1 )) { 
					print "\t", $1;
					if( defined( $2 )) {
						print "\t", $2, "\n";
					}
					else {
						print "\n";
					}
				}
				else { print "\t", $_ ; }
            }
		}
    }
}
sub read_ETS_list{
	my $filename = shift;
	my $regex = '';
	
	open FILE, "<$filename" ;
	while (<FILE>){
		if (/([a-z][a-z][1-9][0-9]?)\s?/i){
			$regex .= lc($1) . '|';
		}
	}
	close FILE;
	$regex =~ s/\|$//;  #remove trailing |
	$regex;
}

=head1 NAME

searchCtl.pl - command to do general searches on Magic Control files

=head1 SYNOPSIS

B<searchCtl> [B<-x (clpt)>] [B<searchString>] < input > output

=head1 DESCRIPTION

I<searchCtl> will search input which it expects to be an exported Internal 
Format Magic Control file for lines that match the provided search string and 
print them to standard output in the format:

ProgramName~~~match

The first (only) argument is the search string
If no search string we'll print all Prog and Task names
If the search string has a parenthesized section we'll just print that
but, otherwise we'll print the whole line that matched the search string.

If there is no search string given as a parameter, we'll print a Program and Task list.

If the searchString is enclosed in parentheses, then only the match is printed, 
otherwise, the whole line is printed.

=back

=head1 ENVIRONMENT

No environment variables are used.

=head1 OPTIONS

=head2 B<-x context>

Specifies context of the search string.  Two contexts are supported  - p and c.

=item B<context := p>  

=item

The search string is a program number to cross reference.  Examples:

=over 5

   searchCtl -x p \"27\"    < FR    cross refs program 27
   searchCtl -x p \"95|96\" < FR    cross refs programs 95 and 96

=back

=item B<context := c>  

=item

The search string is a color number to cross reference. Example:

=over 5

   searchCtl -x c \"[0-9]+\" < FR    cross references all colors.

=back

=item

Sample calls (note how the second example does not need to be quoted):

=over 5

    searchCtl \"COLOR=([0-9]+)\" < /FACT/USER/irag/gen/499.i 
    searchCtl   COLOR=[0-9]+     < /FACT/USER/irag/gen/499.i 
    searchCtl                    < /FACT/USER/irag/gen/499.i 

=back

=head1 AUTHOR

Ira Gershenhorn, Norvin Knight

=head1 HISTORY

=head2 1.0

 21-May-1998 Created by Ira Gershenhorn
 16-Oct-1998 IG added search for specific ETS's.

=item 1. From Norvin Knight's C search program.

=item 2. Execution time is comparable to that of the C program.

=head1 SEE ALSO

none

=cut

sub usage{
    print <<"USAGETEXT";
  usage:  $0 [-x (c|p|t)] [searchString] < input > output

     The first (only) argument is the search string
     ALL SEARCHES ARE CASE INSENSITIVE!!
     If no search string we'll print all Prog and Task names
     If the search string has a parenthesized section we'll just print that
        but, otherwise we'll print the whole line that matched the search string.
    
     If there is no parameter, we'll print a Program and Task list.
    
 Other options:
 -l     => don't use Program: and Task: labels.

 -m     => w for write, r for read, b for both read/write

 -x f   => the search string is a file number; see -m

 -x c   => the search string is a color number to cross reference
		   $0 -x c \"[0-9]+\" <FR    cross references all colors.

 -x p   => the search string is a program number to cross reference
		   $0 -x p \"27\" <FR    cross references program 27
		   $0 -x p \"95|96\" <FR    cross references programs 95 and 96

 -x t   => Print Modification Dates and Times of all tasks. No search string is necessary. 

 -x l   => Print programs,tasks with Logical Names, No search string is necessary. 

 -p     => Print program lines only, ignore tasks for -xt

EXAMPLE calls (note how the second example does not need to be quoted):
      $0 \"COLOR=([0-9]+)\" </FACT/USER/irag/gen/499.i 
      $0  COLOR=[0-9]+    </FACT/USER/irag/gen/499.i 
      $0                  </FACT/USER/irag/gen/499.i 
      $0                  </FACT/USER/irag/gen/499.i 

     prints task modification dates/times
      $0 "^UPD={DATE" < 1.i";
			     
USAGETEXT

    exit( 0 );
}
